package org.bitbucket.bradleysmithllc.etlunit.feature.oracle_database;

import org.bitbucket.bradleysmithllc.etlunit.TestExecutionError;
import org.bitbucket.bradleysmithllc.etlunit.feature.database.BaseDatabaseImplemenation;
import org.bitbucket.bradleysmithllc.etlunit.feature.database.DatabaseConnection;
import org.bitbucket.bradleysmithllc.etlunit.feature.database.DatabaseImplementation;
import org.bitbucket.bradleysmithllc.etlunit.feature.database.NullResultSetClient;
import org.bitbucket.bradleysmithllc.etlunit.feature.database.db.Table;
import org.bitbucket.bradleysmithllc.etlunit.util.IOUtils;
import org.bitbucket.bradleysmithllc.etlunit.util.VelocityUtil;

import java.io.IOException;
import java.net.URL;
import java.sql.Connection;
import java.sql.ResultSet;
import java.util.HashMap;
import java.util.Map;

public class OracleDatabaseImplementation extends BaseDatabaseImplemenation
{
	public String getImplementationId()
	{
		return "oracle";
	}

	public Object processOperationSub(operation op, OperationRequest request) throws UnsupportedOperationException
	{
		switch (op)
		{
			case createDatabase:
				doCreateDatabase(request.getInitializeRequest());
				break;
			case materializeViews:
			{
				try {
					doMaterializeViews(request.getInitializeRequest());
				} catch (IOException e) {
					throw new RuntimeException(e);
				} catch (TestExecutionError testExecutionError) {
					throw new RuntimeException(testExecutionError);
				}
				break;
			}
		}

		return null;
	}

	private void doMaterializeViews(InitializeRequest initializeRequest) throws IOException, TestExecutionError {
		URL url = getClass().getResource("/materializeView.vm");
		final String materialize = IOUtils.readURLToString(url);

		final Map<String, String> context = new HashMap<String, String>();
		// grab the renaming strategy for views
		final DatabaseConnection dc = initializeRequest.getConnection();

		String renStrat = dc.getDatabaseProperties().get("view-rename-strategy");
		if (renStrat == null || renStrat.trim().equals(""))
		{
			renStrat = "_V";
		}

		context.put("view-rename-strategy", renStrat);

		jdbcClient.useResultSetScriptResource(
				dc,
				initializeRequest.getMode(),
				new NullResultSetClient() {
					@Override
					public void next(Connection conn, ResultSet st, DatabaseConnection connection, String mode, int id) throws Exception {
						// issue a rename for the view
						String viewName = st.getString(1);
						applicationLog.info("Found View [" + viewName + "]");
						context.put("view-name", viewName);

						jdbcClient.useResultSetScript(connection, mode, null, VelocityUtil.writeTemplate(materialize, context), id);
					}
				},
				"selectViews.vm"
		);
	}

	private void doCreateDatabase(InitializeRequest initializeRequest)
	{
		Exception[] res = executeScripts(initializeRequest.getConnection(), initializeRequest.getMode(), new String[]{"kill_login", "kill_table_space", "kill_temp_table_space", "create_schema"}, SYSADMIN);

		// make sure the create script did not fail.
		if (res[3] != null)
		{
			throw new IllegalStateException("Error creating schema", res[3]);
		}
	}

	@Override
	public String getJdbcUrl(DatabaseConnection dc, String mode, int id)
	{
		String serviceName = "";

		Map<String, String> databaseProperties = dc.getDatabaseProperties();

		if (databaseProperties != null)
		{
			String oserviceName = databaseProperties.get("service-name");
			if (oserviceName != null)
			{
				serviceName = "/" + oserviceName;
			}
		}

		return "jdbc:oracle:thin:@//" + dc.getServerName() + ":" + String.valueOf(dc.getServerPort() == -1 ? 1521 : dc.getServerPort()) + serviceName;
	}

	@Override
	public Class getJdbcDriverClass()
	{
		return oracle.jdbc.OracleDriver.class;
	}

	@Override
	protected String getLoginNameImpl(DatabaseConnection dc, String mode) {
		String login = IOUtils.hashToLength(getDatabaseName(dc, mode), 28).toUpperCase();
		return login;
	}

	@Override
	public String getPasswordImpl(DatabaseConnection dc, String mode) {
		String pass = IOUtils.hashToLength(getDatabaseName(dc, mode), 28).toUpperCase();
		return pass;
	}

	public String getDefaultSchema(DatabaseConnection databaseConnection, String mode)
	{
		return getLoginName(databaseConnection, mode, DATABASE);
	}

	// all tables are visible since we only see our own schema
	@Override
	public boolean isTableTestVisible(DatabaseConnection dc, String mode, Table table) {
		return table.getType() == Table.type.view || table.getType() == Table.type.table;
	}
}
